module top(
        input           refclk,

        input           rmii_rx_crs_dv,
        input   [ 1:0]  rmii_rxd,
        output          rmii_tx_en,
        output  [ 1:0]  rmii_txd,

        output          rx_error_all_check
   );

reg     [15:0]  rstn_ip_cnt = 16'd0;
reg             rstn_ip_d1 = 0;
reg             rstn_ip_d2 = 0;
reg             rstn_ip = 0;

wire            speedis_100;
wire            duplex_status;

//MII
wire            mii_rx_clk;
wire            mii_rx_dv;
wire            mii_rx_er;
wire    [ 3:0]  mii_rxd;
wire            mii_tx_clk;
wire            mii_tx_en;
wire            mii_tx_er;
wire    [ 3:0]  mii_txd;
wire            mii_col;
wire            mii_crs;

//MAC TX PORT
wire            tx_mac_clk;
wire            tx_mac_valid;
wire    [ 7:0]  tx_mac_data;
wire            tx_mac_last;
wire            tx_mac_error;
wire            tx_mac_ready;

wire            tx_statistics_valid;
//MAC RX PORT
wire            rx_mac_clk;
wire            rx_mac_valid;
wire    [ 7:0]  rx_mac_data;
wire            rx_mac_last;
wire            rx_mac_error;

wire            rx_statistics_valid;
wire    [26:0]  rx_statistics_vector;

reg             cycle_all_time;
wire            rx_running;
reg             rx_running_d1;
wire    [31:0]  rx_cnt;

always @(posedge refclk) begin
    if(rstn_ip_cnt == 16'hffff) begin
        rstn_ip_cnt  <=  rstn_ip_cnt;
    end else begin
        rstn_ip_cnt  <=  rstn_ip_cnt + 16'd1;
    end
end

always @(posedge refclk) begin
    rstn_ip_d1    <=  rstn_ip_cnt[15];
    rstn_ip_d2    <=  rstn_ip_d1;
    rstn_ip       <=  rstn_ip_d2;
end

assign  speedis_100     =   1'b1;   //1:100M           0:10M
assign  duplex_status   =   1'b0;   //1:half duplex    0:Full duplex

MII_to_RMII_Top u_mii_to_rmii_top (
		.refclk                 (refclk			        ),
		.rstn			        (rstn_ip 			    ),

		.speedis_100	        (speedis_100            ),

		.rmii_rx_crs_dv         (rmii_rx_crs_dv	        ),
		.rmii_rx_er	            (1'b0   		        ),
		.rmii_rxd 	            (rmii_rxd		        ),
		.rmii_tx_en	            (rmii_tx_en		        ),
		.rmii_txd	            (rmii_txd		        ),

		.mii_rx_clk	            (mii_rx_clk		        ),
		.mii_rx_dv	            (mii_rx_dv		        ),
		.mii_rx_er	            (mii_rx_er		        ),
		.mii_rxd	            (mii_rxd		        ),
		.mii_tx_clk	            (mii_tx_clk		        ),
		.mii_tx_en	            (mii_tx_en		        ),
		.mii_tx_er	            (mii_tx_er		        ),
		.mii_txd	            (mii_txd		        ),
		.mii_col	            (mii_col		        ),
		.mii_crs	            (mii_crs		        )
	);

Triple_Speed_Ethernet_MAC_Top u_Triple_Speed_Ethernet_MAC_Top(
        .mii_rx_clk             (mii_rx_clk             ), //input mii_rx_clk
        .mii_rx_dv              (mii_rx_dv              ), //input mii_rx_dv
        .mii_rx_er              (mii_rx_er              ), //input mii_rx_er
        .mii_rxd                (mii_rxd                ), //input [3:0] mii_rxd
        .mii_tx_clk             (mii_tx_clk             ), //input mii_tx_clk
        .mii_tx_en              (mii_tx_en              ), //output mii_tx_en
        .mii_tx_er              (mii_tx_er              ), //output mii_tx_er
        .mii_txd                (mii_txd                ), //output [3:0] mii_txd
        .mii_col                (mii_col                ), //input mii_col
        .mii_crs                (mii_crs                ), //input mii_crs
        .duplex_status          (duplex_status          ), //input duplex_status
        .rstn                   (rstn_ip                ), //input rstn
        .rx_mac_clk             (rx_mac_clk             ), //output rx_mac_clk
        .rx_mac_valid           (rx_mac_valid           ), //output rx_mac_valid
        .rx_mac_data            (rx_mac_data            ), //output [7:0] rx_mac_data
        .rx_mac_last            (rx_mac_last            ), //output rx_mac_last
        .rx_mac_error           (rx_mac_error           ), //output rx_mac_error
        .rx_statistics_valid    (rx_statistics_valid    ), //output rx_statistics_valid
        .rx_statistics_vector   (rx_statistics_vector   ), //output [26:0] rx_statistics_vector
        .tx_mac_clk             (tx_mac_clk             ), //output tx_mac_clk
        .tx_mac_valid           (tx_mac_valid           ), //input tx_mac_valid
        .tx_mac_data            (tx_mac_data            ), //input [7:0] tx_mac_data
        .tx_mac_last            (tx_mac_last            ), //input tx_mac_last
        .tx_mac_error           (tx_mac_error           ), //input tx_mac_error
        .tx_mac_ready           (tx_mac_ready           ), //output tx_mac_ready
        .tx_collision           (                       ), //output tx_collision
        .tx_retransmit          (                       ), //output tx_retransmit
        .tx_statistics_valid    (tx_statistics_valid    ), //output tx_statistics_valid
        .tx_statistics_vector   (                       ), //output [28:0] tx_statistics_vector
        .rx_fcs_fwd_ena         (1'd0                   ), //input rx_fcs_fwd_ena
        .rx_jumbo_ena           (1'd0                   ), //input rx_jumbo_ena
        .rx_pause_req           (                       ), //output rx_pause_req
        .rx_pause_val           (                       ), //output [15:0] rx_pause_val
        .tx_fcs_fwd_ena         (1'd0                   ), //input tx_fcs_fwd_ena
        .tx_ifg_delay_ena       (1'd0                   ), //input tx_ifg_delay_ena
        .tx_ifg_delay           (8'd0                   ), //input [7:0] tx_ifg_delay
        .tx_pause_req           (1'd0                   ), //input tx_pause_req
        .tx_pause_val           (16'd0                  ), //input [15:0] tx_pause_val
        .tx_pause_source_addr   (48'd0                  ), //input [47:0] tx_pause_source_addr
        .clk                    (1'b0                   ), //input clk
        .miim_phyad             (5'd0                   ), //input [4:0] miim_phyad
        .miim_regad             (5'd0                   ), //input [4:0] miim_regad
        .miim_wrdata            (16'd0                  ), //input [15:0] miim_wrdata
        .miim_wren              (1'b0                   ), //input miim_wren
        .miim_rden              (1'b0                   ), //input miim_rden
        .miim_rddata            (                       ), //output [15:0] miim_rddata
        .miim_rddata_valid      (                       ), //output miim_rddata_valid
        .miim_busy              (                       ), //output miim_busy
        .mdc                    (                       ), //output mdc
        .mdio_in                (1'b0                   ), //input mdio_in
        .mdio_out               (                       ), //output mdio_out
        .mdio_oen               (                       ) //output mdio_oen
    );

always @(posedge rx_mac_clk or negedge rstn_ip) begin
    if(~rstn_ip) begin
        rx_running_d1   <=  1'b0;
    end else begin
        rx_running_d1   <=  rx_running;
    end
end

always @(posedge rx_mac_clk or negedge rstn_ip) begin
    if(~rstn_ip) begin
        cycle_all_time  <=  1'b0;
    end else begin
        if(cycle_all_time) begin
            if(rx_running && ~rx_running_d1) begin
                cycle_all_time  <=  ~(rx_cnt > 32'h00000fff);
            end
        end else begin
            cycle_all_time  <=  ~(rx_cnt > 32'h00000fff);
        end
    end
end

mac_tx_model u_mac_tx_model(
        .tx_mac_clk             (tx_mac_clk             ),
        .tx_mac_rst_n           (rstn_ip                ),
        
    //CONTROL PORT
        .i_cycle_all_time       (cycle_all_time         ),

        .tx_running             (                       ),

    //IP TX PORT
        .tx_mac_valid           (tx_mac_valid           ),
        .tx_mac_data            (tx_mac_data            ),
        .tx_mac_last            (tx_mac_last            ),
        .tx_mac_error           (tx_mac_error           ),
        .tx_mac_ready           (tx_mac_ready           ),

        .tx_statistics_valid    (tx_statistics_valid    ),
        .tx_cnt_full            (                       ),
        .tx_cnt                 (                       )
    );

mac_rx_model u_mac_rx_model(
        .rx_mac_clk             (rx_mac_clk             ),
        .rx_mac_rst_n           (rstn_ip                ),
                
    //CONTROL PORT
        .i_cycle_all_time       (cycle_all_time         ),

        .rx_running             (rx_running             ),
    //IP TX PORT
        .rx_mac_valid           (rx_mac_valid           ),
        .rx_mac_data            (rx_mac_data            ),
        .rx_mac_last            (rx_mac_last            ),
        .rx_mac_error           (rx_mac_error           ),

        .rx_statistics_valid    (rx_statistics_valid    ),
        .rx_statistics_vector   (rx_statistics_vector   ),

    //CHECK RESULT
        .rx_error_all_check     (rx_error_all_check     ),
        .rx_cnt_full            (                       ),
        .rx_cnt                 (rx_cnt                 )
    );

endmodule